/****************************************************************/
/* Widget Support Library WSL */
/****************************************************************/

#include <stdio.h>
#include <string.h>

#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

/* Motif Widget Set */
#include <Xm/Xm.h>
#include <Xm/DialogS.h>

#include <wsl.h>

typedef struct {
    Widget w;
    int name;
    unsigned int key;
} WidgetEntry;			/* widget to name association */

#define MAX_WIDGETS	1024
#define MAX_OTHER	256
static WidgetEntry WidgetDB[MAX_WIDGETS];
static WidgetEntry OtherDB[MAX_OTHER];
static int lastWidget = 0;
static int lastOther = 0;


void WslWarning (msg)
     char *msg;
{
    puts (msg);
} /* end WslWarning */




/*------------------------------------------------------------------------*/
/* Function Name:	WslStoreWidget
/* 
/* Description:		Store widget id and its name in a database. Most
/*			useful for stashing away MrmNcreateCallback widget
/*			id's.
/*
/* Comments:		We are assuming that the static declaration if the
/*			WidgetDB variable forces the compiler to initialize
/*			the data area to 0.
/*
/* Arguments:		name		widget name
/*			w		widget id
/*			key		a unique key
/*
/* Return Value:	WSL_SUCCESS	if ok
/*			WSL_FAILURE	if widget could not be stored
/* 
/* Author:		Martin J. Schedlbauer
/*
/* Revision History:
/* 
/*------------------------------------------------------------------------*/

int WslStoreWidget (w, name, key)
     Widget w;
     int name;
     unsigned int key;
{
    int val = WSL_FAILURE;
    Boolean found = FALSE;
    int slot;

    /* look through the array starting at last_pos for a free slot */
    slot = lastWidget;
    do {
	if (WidgetDB[slot].w == NULL) {
	    WidgetDB[slot].w = w;
	    WidgetDB[slot].name = name;
	    WidgetDB[slot].key = key;
	    lastWidget = (slot + 1) % MAX_WIDGETS;
	    found = TRUE;
	    break;
	} else {
	    slot = (slot + 1) % MAX_WIDGETS;
	}

	/* check if we wrapped around without finding a slot */
	if (slot == lastWidget) {
            WslWarning ("WslStoreWidget: out of memory");
	    exit (1);
	}
    } while (!found);

    return WSL_SUCCESS;
} /* end WslStoreWidget */




/*------------------------------------------------------------------------*/
/* Function Name:	WslRetrieveWidget
/* 
/* Description:		Retrieve a previously stored widget by name and key.
/*
/* Arguments:		name		widget name
/*			key		widget key    
/*
/* Return Value:	widget id	if ok
/*			WSL_NULL	if widget could not be found
/* 
/* Author:		Martin J. Schedlbauer
/*
/* Revision History:
/* 
/*------------------------------------------------------------------------*/

Widget WslRetrieveWidget (name, key)
     int name;
     unsigned int key;
{
    int i;
    char buf[80];

    for (i = 0; i < MAX_WIDGETS; ++i)
      if (WidgetDB[i].name == name && WidgetDB[i].key == key) {
	  return WidgetDB[i].w;
	  break;
      }

    sprintf (buf, "WslRetrieveWidget: widget '%d' not found\n", name);
    WslWarning (buf);

    return NULL;
} /* end WslRetrieveWidget */




Widget WslGetShell (w)
     Widget w;
{
    Widget shell_widget;
    WidgetClass class;

    shell_widget = w;
    do {
	shell_widget = XtParent (shell_widget);
	class = XtClass (shell_widget);
    } while ((class != topLevelShellWidgetClass) && 
             (class != applicationShellWidgetClass) &&
	     (class != xmDialogShellWidgetClass));
    return shell_widget;
} /* end WslGetShell */




/*------------------------------------------------------------------------*/
/* Function Name:	WslUnregisterWidgets
/* 
/* Description:		Remove the widgets from the database assoicated
/*			with the passed key.
/*
/* Arguments:		key		a unique key
/*
/* Return Value:	number of widgets unregistered
/* 
/* Author:		Martin J. Schedlbauer
/*
/* Revision History:
/* 
/*------------------------------------------------------------------------*/

int WslUnregisterWidgets (key)
     unsigned int key;
{
    int slot, count;

    for (slot = 0, count = 0; slot < MAX_WIDGETS; ++slot) {
	if (WidgetDB[slot].key == key) {
	    WidgetDB[slot].w = NULL;
	    WidgetDB[slot].name = 0;
	    WidgetDB[slot].key = 0;
	    ++count;
	}
    }

    if (count == 0)
      WslWarning ("WslUnregisterWidgets: no widget unregistered");

    return count;
} /* end WslUnregisterWidgets */




/*------------------------------------------------------------------------*/
/* Function Name:	WslDeleteWidget
/* 
/* Description:		Remove the widget from the database 
/*			associated with the passed key.
/*
/* Arguments:		key		a unique key
/*
/* Return Value:	WSL_SUCCESS or WSL_FAILURE
/* 
/* Author:		Martin J. Schedlbauer
/*
/* Revision History:
/* 
/*------------------------------------------------------------------------*/

int WslDeleteWidget (name, key)
     int name;
     unsigned int key;
{
    int slot, count;

    for (slot = 0, count = 0; slot < MAX_WIDGETS; ++slot) {
	if (WidgetDB[slot].key == key && WidgetDB[slot].name == name) {
	    WidgetDB[slot].w = NULL;
	    WidgetDB[slot].name = 0;
	    WidgetDB[slot].key = 0;
	    return WSL_SUCCESS;
	}
    }

    return WSL_FAILURE;
} /* end WslDeleteWidget */




/*------------------------------------------------------------------------*/
/* Function Name:	WslDeleteStashedWidget
/* 
/* Description:		Remove the widget from the "other" database 
/*			associated with the passed key.
/*
/* Arguments:		key		a unique key
/*
/* Return Value:	WSL_SUCCESS or WSL_FAILURE
/* 
/* Author:		Martin J. Schedlbauer
/*
/* Revision History:
/* 
/*------------------------------------------------------------------------*/

int WslDeleteStashedWidget (name, key)
     int name;
     unsigned int key;
{
    int slot, count;

    for (slot = 0, count = 0; slot < MAX_OTHER; ++slot) {
	if (OtherDB[slot].key == key && OtherDB[slot].name == name) {
	    OtherDB[slot].w = NULL;
	    OtherDB[slot].name = 0;
	    OtherDB[slot].key = 0;
	    return WSL_SUCCESS;
	}
    }

    return WSL_FAILURE;
} /* end WslDeleteStashedWidget */




/*------------------------------------------------------------------------*/
/* Function Name:	WslRetrieveName
/* 
/* Description:		Return the name of the widget with the given key.
/*
/* Arguments:		key		a unique key
/*			w		widget id   
/*
/* Return Value:	widget name
/* 
/* Author:		Martin J. Schedlbauer
/*
/* Revision History:
/* 
/*------------------------------------------------------------------------*/

int WslRetrieveName (w, key)
     Widget w;
     unsigned int key;
{
    int slot, count;

    for (slot = 0, count = 0; slot < MAX_WIDGETS; ++slot) {
	if (WidgetDB[slot].w == w && WidgetDB[slot].key == key) {
	    return WidgetDB[slot].name;
	}
    }

    return WSL_FAILURE;
} /* end WslRetrieveName */




/*------------------------------------------------------------------------*/
/* Function Name:	WslStashWidget
/* 
/* Description:		Store widget id and its name in a database. Most
/*			useful for stashing away selected widget id's
/*			that can't conflict with other widgets.
/*
/* Comments:		We are assuming that the static declaration of the
/*			OtherDB variable forces the compiler to initialize
/*			the data area to 0.
/*
/* Arguments:		name		widget name
/*			w		widget id
/*			key		a unique key
/*
/* Return Value:	WSL_SUCCESS	if ok
/*			WSL_FAILURE	if widget could not be stored
/* 
/* Author:		Martin J. Schedlbauer
/*
/* Revision History:
/* 
/*------------------------------------------------------------------------*/

int WslStashWidget (w, name, key)
     Widget w;
     int name;
     unsigned int key;
{
    int val = WSL_FAILURE;
    Boolean found = FALSE;
    int slot;

    /* look through the array starting at last_pos for a free slot */
    slot = lastOther;
    do {
	if (OtherDB[slot].w == NULL) {
	    OtherDB[slot].w = w;
	    OtherDB[slot].name = name;
	    OtherDB[slot].key = key;
	    lastOther = (slot + 1) % MAX_OTHER;
	    found = TRUE;
	    break;
	} else {
	    slot = (slot + 1) % MAX_OTHER;
	}

	/* check if we wrapped around without finding a slot */
	if (slot == lastOther) {
            WslWarning ("WslStashWidget: out of memory");
	    exit (1);
	}
    } while (!found);

    return WSL_SUCCESS;
} /* end WslStashWidget */




/*------------------------------------------------------------------------*/
/* Function Name:	WslRetrieveStashedWidget
/* 
/* Description:		Retrieve a previously stored widget by name and key.
/*
/* Arguments:		name		widget name
/*			key		widget key    
/*
/* Return Value:	widget id	if ok
/*			WSL_NULL	if widget could not be found
/* 
/* Author:		Martin J. Schedlbauer
/*
/* Revision History:
/* 
/*------------------------------------------------------------------------*/

Widget WslRetrieveStashedWidget (name, key)
     int name;
     unsigned int key;
{
    int i;
    char buf[80];

    for (i = 0; i < MAX_OTHER; ++i)
      if (OtherDB[i].name == name && OtherDB[i].key == key) {
	  return OtherDB[i].w;
	  break;
      }

    sprintf (buf, "WslRetrieveStashedWidget: widget '%d' not found\n", name);
    WslWarning (buf);

    return NULL;
} /* end WslRetrieveStashedWidget */




/*------------------------------------------------------------------------*/
/* Function Name:	WslUnregisterStashedWidget
/* 
/* Description:		Remove the widgets from the "other" database 
/*			associated with the passed key.
/*
/* Arguments:		key		a unique key
/*
/* Return Value:	number of widgets unregistered
/* 
/* Author:		Martin J. Schedlbauer
/*
/* Revision History:
/* 
/*------------------------------------------------------------------------*/

int WslUnregisterStashedWidgets (key)
     unsigned int key;
{
    int slot, count;

    for (slot = 0, count = 0; slot < MAX_OTHER; ++slot) {
	if (OtherDB[slot].key == key) {
	    OtherDB[slot].w = NULL;
	    OtherDB[slot].name = 0;
	    OtherDB[slot].key = 0;
	    ++count;
	}
    }

    if (count == 0)
      WslWarning ("WslUnregisterStashedWidgets: no widget unregistered");

    return count;
} /* end WslUnregisterStashedWidgets */
